@@ -248,7 +248,9 @@ class IntentAPI(StoreUpdatingAPI):
             await self.state_store.joined(room_id, user_id)
         except MatrixRequestError as e:
             # TODO remove this once MSC3848 is released and minimum spec version is bumped
-            if e.errcode == "M_FORBIDDEN" and "is already in the room" in e.message:
+            if e.errcode == "M_FORBIDDEN" and (
+                "already in the room" in e.message or "is already joined to room" in e.message
+            ):
                 await self.state_store.joined(room_id, user_id)
             else:
                 raise
@@ -405,15 +407,10 @@ class IntentAPI(StoreUpdatingAPI):
     async def set_typing(
         self,
         room_id: RoomID,
-        is_typing: bool = True,
-        timeout: int = 5000,
-        ignore_cache: bool = False,
+        timeout: int = 0,
     ) -> None:
         await self.ensure_joined(room_id)
-        if not ignore_cache and is_typing == self.state_store.is_typing(room_id, self.mxid):
-            return
-        await super().set_typing(room_id, timeout if is_typing else 0)
-        self.state_store.set_typing(room_id, self.mxid, is_typing, timeout)
+        await super().set_typing(room_id, timeout)
 
     async def error_and_leave(
         self, room_id: RoomID, text: str | None = None, html: str | None = None
@@ -501,6 +498,7 @@ class IntentAPI(StoreUpdatingAPI):
         events: Iterable[BatchSendEvent],
         state_events_at_start: Iterable[BatchSendStateEvent] = (),
         beeper_new_messages: bool = False,
+        beeper_mark_read_by: UserID | None = None,
     ) -> BatchSendResponse:
         """
         Send a batch of historical events into a room. See `MSC2716`_ for more info.
@@ -530,6 +528,8 @@ class IntentAPI(StoreUpdatingAPI):
             query["batch_id"] = batch_id
         if beeper_new_messages:
             query["com.beeper.new_messages"] = "true"
+        if beeper_mark_read_by:
+            query["com.beeper.mark_read_by"] = beeper_mark_read_by
         resp = await self.api.request(
             Method.POST,
             path,
@@ -541,6 +541,12 @@ class IntentAPI(StoreUpdatingAPI):
         )
         return BatchSendResponse.deserialize(resp)
 
+    async def beeper_delete_room(self, room_id: RoomID) -> None:
+        versions = await self.versions()
+        if not versions.supports("com.beeper.room_yeeting"):
+            raise RuntimeError("Homeserver does not support yeeting rooms")
+        await self.api.request(Method.POST, Path.unstable["com.beeper.yeet"].rooms[room_id].delete)
+
     # endregion
     # region Ensure functions
 
