@@ -16,14 +16,18 @@ from aiohttp import web
 
 from mautrix.types import (
     JSON,
+    ASToDeviceEvent,
+    DeviceID,
     DeviceLists,
     DeviceOTKCount,
     EphemeralEvent,
     Event,
+    EventType,
     RoomAlias,
     SerializerError,
     UserID,
 )
+from mautrix.util import background_task
 
 HandlerFunc = Callable[[Event], Awaitable]
 
@@ -34,17 +38,25 @@ class AppServiceServerMixin:
 
     hs_token: str
     ephemeral_events: bool
+    encryption_events: bool
 
     query_user: Callable[[UserID], JSON]
     query_alias: Callable[[RoomAlias], JSON]
 
     transactions: set[str]
     event_handlers: list[HandlerFunc]
+    to_device_handler: HandlerFunc | None
+    otk_handler: Callable[[dict[UserID, dict[DeviceID, DeviceOTKCount]]], Awaitable] | None
+    device_list_handler: Callable[[DeviceLists], Awaitable] | None
 
-    def __init__(self, ephemeral_events: bool = False) -> None:
+    def __init__(self, ephemeral_events: bool = False, encryption_events: bool = False) -> None:
         self.transactions = set()
         self.event_handlers = []
+        self.to_device_handler = None
+        self.otk_handler = None
+        self.device_list_handler = None
         self.ephemeral_events = ephemeral_events
+        self.encryption_events = encryption_events
 
         async def default_query_handler(_):
             return None
@@ -150,8 +162,11 @@ class AppServiceServerMixin:
     async def _http_handle_transaction(self, request: web.Request) -> web.Response:
         transaction_id, data = await self._read_transaction_header(request)
 
+        txn_content_log = []
         try:
             events = data.pop("events")
+            if events:
+                txn_content_log.append(f"{len(events)} PDUs")
         except KeyError:
             raise web.HTTPBadRequest(
                 content_type="application/json",
@@ -160,23 +175,46 @@ class AppServiceServerMixin:
                 ),
             )
 
-        ephemeral = (
-            self._get_with_fallback(data, "ephemeral", "de.sorunome.msc2409")
-            if self.ephemeral_events
-            else None
-        )
-        device_lists = DeviceLists.deserialize(
-            self._get_with_fallback(data, "device_lists", "org.matrix.msc3202")
-        )
-        otk_counts = {
-            user_id: {
-                device_id: DeviceOTKCount.deserialize(count)
-                for device_id, count in devices.items()
+        if self.ephemeral_events:
+            ephemeral = self._get_with_fallback(data, "ephemeral", "de.sorunome.msc2409")
+            if ephemeral:
+                txn_content_log.append(f"{len(ephemeral)} EDUs")
+        else:
+            ephemeral = None
+        if self.encryption_events:
+            to_device = self._get_with_fallback(data, "to_device", "de.sorunome.msc2409")
+            device_lists = DeviceLists.deserialize(
+                self._get_with_fallback(data, "device_lists", "org.matrix.msc3202")
+            )
+            otk_counts = {
+                user_id: {
+                    device_id: DeviceOTKCount.deserialize(count)
+                    for device_id, count in devices.items()
+                }
+                for user_id, devices in self._get_with_fallback(
+                    data, "device_one_time_keys_count", "org.matrix.msc3202", default={}
+                ).items()
             }
-            for user_id, devices in self._get_with_fallback(
-                data, "device_one_time_keys_count", "org.matrix.msc3202", default={}
-            ).items()
-        }
+            if to_device:
+                txn_content_log.append(f"{len(to_device)} to-device events")
+            if device_lists.changed:
+                txn_content_log.append(f"{len(device_lists.changed)} device list changes")
+            if otk_counts:
+                txn_content_log.append(
+                    f"{sum(len(vals) for vals in otk_counts.values())} OTK counts"
+                )
+        else:
+            otk_counts = {}
+            device_lists = None
+            to_device = None
+
+        if len(txn_content_log) > 2:
+            txn_content_log = [", ".join(txn_content_log[:-1]), txn_content_log[-1]]
+        if not txn_content_log:
+            txn_description = "nothing?"
+        else:
+            txn_description = " and ".join(txn_content_log)
+        self.log.debug(f"Handling transaction {transaction_id} with {txn_description}")
 
         try:
             output = await self.handle_transaction(
@@ -184,12 +222,15 @@ class AppServiceServerMixin:
                 events=events,
                 extra_data=data,
                 ephemeral=ephemeral,
+                to_device=to_device,
                 device_lists=device_lists,
-                device_otk_count=otk_counts,
+                otk_counts=otk_counts,
             )
         except Exception:
             self.log.exception("Exception in transaction handler")
             output = None
+        finally:
+            self.log.debug(f"Finished handling transaction {transaction_id}")
 
         self.transactions.add(transaction_id)
 
@@ -197,6 +238,11 @@ class AppServiceServerMixin:
 
     @staticmethod
     def _fix_prev_content(raw_event: JSON) -> None:
+        try:
+            if raw_event["unsigned"] is None:
+                del raw_event["unsigned"]
+        except KeyError:
+            pass
         try:
             raw_event["unsigned"]["prev_content"]
         except KeyError:
@@ -212,16 +258,37 @@ class AppServiceServerMixin:
         events: list[JSON],
         extra_data: JSON,
         ephemeral: list[JSON] | None = None,
-        device_otk_count: dict[UserID, dict[DeviceID, DeviceOTKCount]] | None = None,
+        to_device: list[JSON] | None = None,
+        otk_counts: dict[UserID, dict[DeviceID, DeviceOTKCount]] | None = None,
         device_lists: DeviceLists | None = None,
     ) -> JSON:
+        for raw_td in to_device or []:
+            try:
+                td = ASToDeviceEvent.deserialize(raw_td)
+            except SerializerError:
+                self.log.exception("Failed to deserialize to-device event %s", raw_td)
+            else:
+                try:
+                    await self.to_device_handler(td)
+                except:
+                    self.log.exception("Exception in Matrix to-device event handler")
+        if device_lists and self.device_list_handler:
+            try:
+                await self.device_list_handler(device_lists)
+            except Exception:
+                self.log.exception("Exception in Matrix device list change handler")
+        if otk_counts and self.otk_handler:
+            try:
+                await self.otk_handler(otk_counts)
+            except Exception:
+                self.log.exception("Exception in Matrix OTK count handler")
         for raw_edu in ephemeral or []:
             try:
                 edu = EphemeralEvent.deserialize(raw_edu)
             except SerializerError:
                 self.log.exception("Failed to deserialize ephemeral event %s", raw_edu)
             else:
-                self.handle_matrix_event(edu)
+                self.handle_matrix_event(edu, ephemeral=True)
         for raw_event in events:
             try:
                 self._fix_prev_content(raw_event)
@@ -232,10 +299,13 @@ class AppServiceServerMixin:
                 self.handle_matrix_event(event)
         return {}
 
-    def handle_matrix_event(self, event: Event) -> None:
-        if event.type.is_state and event.state_key is None:
-            self.log.debug(f"Not sending {event.event_id} to handlers: expected state_key.")
-            return
+    def handle_matrix_event(self, event: Event, ephemeral: bool = False) -> None:
+        if ephemeral:
+            event.type = event.type.with_class(EventType.Class.EPHEMERAL)
+        elif getattr(event, "state_key", None) is not None:
+            event.type = event.type.with_class(EventType.Class.STATE)
+        else:
+            event.type = event.type.with_class(EventType.Class.MESSAGE)
 
         async def try_handle(handler_func: HandlerFunc):
             try:
@@ -244,7 +314,8 @@ class AppServiceServerMixin:
                 self.log.exception("Exception in Matrix event handler")
 
         for handler in self.event_handlers:
-            asyncio.create_task(try_handle(handler))
+            # TODO add option to handle events synchronously
+            background_task.create(try_handle(handler))
 
     def matrix_event_handler(self, func: HandlerFunc) -> HandlerFunc:
         self.event_handlers.append(func)
