@@ -5,8 +5,10 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from __future__ import annotations
 
-from typing import Any
+from typing import Any, ClassVar
 from abc import ABC
+import json
+import os
 import re
 import secrets
 import time
@@ -21,15 +23,40 @@ from mautrix.util.config import (
 
 
 class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
+    env_prefix: str | None = None
     registration_path: str
     _registration: dict | None
     _check_tokens: bool
+    env: dict[str, Any]
 
-    def __init__(self, path: str, registration_path: str, base_path: str) -> None:
+    def __init__(
+        self, path: str, registration_path: str, base_path: str, env_prefix: str | None = None
+    ) -> None:
         super().__init__(path, base_path)
         self.registration_path = registration_path
         self._registration = None
         self._check_tokens = True
+        self.env = {}
+        if not self.env_prefix:
+            self.env_prefix = env_prefix
+        if self.env_prefix:
+            env_prefix = f"{self.env_prefix}_"
+            for key, value in os.environ.items():
+                if not key.startswith(env_prefix):
+                    continue
+                key = key.removeprefix(env_prefix)
+                if value.startswith("json::"):
+                    value = json.loads(value.removeprefix("json::"))
+                self.env[key] = value
+
+    def __getitem__(self, item: str) -> Any:
+        if self.env:
+            try:
+                sanitized_item = item.replace(".", "_").replace("[", "").replace("]", "").upper()
+                return self.env[sanitized_item]
+            except KeyError:
+                pass
+        return super().__getitem__(item)
 
     def save(self) -> None:
         super().save()
@@ -45,6 +72,7 @@ class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
     def forbidden_defaults(self) -> list[ForbiddenDefault]:
         return [
             ForbiddenDefault("homeserver.address", "https://example.com"),
+            ForbiddenDefault("homeserver.address", "https://matrix.example.com"),
             ForbiddenDefault("homeserver.domain", "example.com"),
         ] + (
             [
@@ -74,6 +102,10 @@ class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
         copy("homeserver.status_endpoint")
         copy("homeserver.message_send_checkpoint_endpoint")
         copy("homeserver.async_media")
+        if self.get("homeserver.asmux", False):
+            helper.base["homeserver.software"] = "asmux"
+        else:
+            copy("homeserver.software")
 
         copy("appservice.address")
         copy("appservice.hostname")
@@ -105,6 +137,7 @@ class BaseBridgeConfig(BaseFileConfig, BaseValidatableConfig, ABC):
         copy("bridge.encryption.allow")
         copy("bridge.encryption.default")
         copy("bridge.encryption.require")
+        copy("bridge.encryption.appservice")
         copy("bridge.encryption.verification_levels.receive")
         copy("bridge.encryption.verification_levels.send")
         copy("bridge.encryption.verification_levels.share")
