@@ -5,6 +5,7 @@
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 from __future__ import annotations
 
+from collections import defaultdict
 import asyncio
 import logging
 import sys
@@ -56,7 +57,7 @@ from mautrix.types import (
     Version,
     VersionsResponse,
 )
-from mautrix.util import markdown
+from mautrix.util import background_task, markdown
 from mautrix.util.logging import TraceLogger
 from mautrix.util.message_send_checkpoint import (
     CHECKPOINT_TYPES,
@@ -143,6 +144,7 @@ class BaseMatrixHandler:
     media_config: MediaRepoConfig
     versions: VersionsResponse
     minimum_spec_version: Version = SpecVersions.V11
+    room_locks: dict[str, asyncio.Lock]
 
     user_id_prefix: str
     user_id_suffix: str
@@ -159,6 +161,7 @@ class BaseMatrixHandler:
         self.media_config = MediaRepoConfig(upload_size=50 * 1024 * 1024)
         self.versions = VersionsResponse.deserialize({"versions": ["v1.3"]})
         self.az.matrix_event_handler(self.int_handle_event)
+        self.room_locks = defaultdict(asyncio.Lock)
 
         self.e2ee = None
         self.require_e2ee = False
@@ -207,6 +210,14 @@ class BaseMatrixHandler:
                 self.minimum_spec_version,
             )
             sys.exit(18)
+        if self.bridge.homeserver_software.is_hungry and not self.versions.supports(
+            "com.beeper.hungry"
+        ):
+            self.log.fatal(
+                "The config claims the homeserver is hungryserv, "
+                "but the /versions response didn't confirm it"
+            )
+            sys.exit(18)
 
     async def wait_for_connection(self) -> None:
         self.log.info("Ensuring connectivity to homeserver")
@@ -374,7 +385,7 @@ class BaseMatrixHandler:
             members = await intent.get_room_members(room_id)
         except MatrixError:
             self.log.exception(f"Failed to get state after joining {room_id} as {intent.mxid}")
-            asyncio.create_task(intent.leave_room(room_id, reason="Internal error"))
+            background_task.create(intent.leave_room(room_id, reason="Internal error"))
             return
         if create_evt.type == RoomType.SPACE:
             await self.handle_puppet_space_invite(room_id, puppet, invited_by, evt)
@@ -392,19 +403,20 @@ class BaseMatrixHandler:
             await intent.leave_room(room_id, reason="You're not allowed to invite this ghost.")
             return
 
-        portal = await self.bridge.get_portal(room_id)
-        if portal:
-            try:
-                await portal.handle_matrix_invite(invited_by, puppet)
-            except br.RejectMatrixInvite as e:
-                await intent.leave_room(room_id, reason=e.message)
-            except br.IgnoreMatrixInvite:
-                pass
+        async with self.room_locks[room_id]:
+            portal = await self.bridge.get_portal(room_id)
+            if portal:
+                try:
+                    await portal.handle_matrix_invite(invited_by, puppet)
+                except br.RejectMatrixInvite as e:
+                    await intent.leave_room(room_id, reason=e.message)
+                except br.IgnoreMatrixInvite:
+                    pass
+                else:
+                    await intent.join_room(room_id)
+                return
             else:
-                await intent.join_room(room_id)
-            return
-        else:
-            await self.handle_puppet_nonportal_invite(room_id, puppet, invited_by, evt)
+                await self.handle_puppet_nonportal_invite(room_id, puppet, invited_by, evt)
 
     async def handle_invite(
         self, room_id: RoomID, user_id: UserID, invited_by: br.BaseUser, evt: StateEvent
@@ -464,23 +476,11 @@ class BaseMatrixHandler:
         )
 
     async def accept_bot_invite(self, room_id: RoomID, inviter: br.BaseUser) -> None:
-        tries = 0
-        while tries < 5:
-            try:
-                await self.az.intent.join_room(room_id)
-                break
-            except (IntentError, MatrixError):
-                tries += 1
-                wait_for_seconds = (tries + 1) * 10
-                if tries < 5:
-                    self.log.exception(
-                        f"Failed to join room {room_id} with bridge bot, "
-                        f"retrying in {wait_for_seconds} seconds..."
-                    )
-                    await asyncio.sleep(wait_for_seconds)
-                else:
-                    self.log.exception(f"Failed to join room {room_id}, giving up.")
-                    return
+        try:
+            await self.az.intent.join_room(room_id)
+        except Exception:
+            self.log.exception(f"Failed to join room {room_id} as bridge bot")
+            return
 
         if not await self.allow_command(inviter):
             await self.send_permission_error(room_id)
@@ -587,7 +587,6 @@ class BaseMatrixHandler:
                 error=msg,
                 message=err.human_message if err else None,
             )
-            status_content.fill_legacy_booleans()
             await self.az.intent.send_message_event(
                 evt.room_id, EventType.BEEPER_MESSAGE_STATUS, status_content
             )
@@ -788,7 +787,7 @@ class BaseMatrixHandler:
         try:
             decrypted = await self.e2ee.decrypt(evt, wait_session_timeout=3)
         except SessionNotFound as e:
-            await self._handle_encrypted_wait(evt, e, wait=6)
+            await self._handle_encrypted_wait(evt, e, wait=22)
         except DecryptionError as e:
             self.log.warning(f"Failed to decrypt {evt.event_id}: {e}")
             self.log.trace("%s decryption traceback:", evt.event_id, exc_info=True)
@@ -803,7 +802,7 @@ class BaseMatrixHandler:
             f"Couldn't find session {err.session_id} trying to decrypt {evt.event_id},"
             " waiting even longer"
         )
-        asyncio.create_task(
+        background_task.create(
             self.e2ee.crypto.request_room_key(
                 evt.room_id,
                 evt.content.sender_key,
@@ -880,7 +879,7 @@ class BaseMatrixHandler:
             info=str(err) if err else None,
             retry_num=retry_num,
         )
-        asyncio.create_task(checkpoint.send(endpoint, self.az.as_token, self.log))
+        background_task.create(checkpoint.send(endpoint, self.az.as_token, self.log))
 
     allowed_event_classes: tuple[type, ...] = (
         MessageEvent,
