@@ -30,6 +30,7 @@ from mautrix.types import (
     TextMessageEventContent,
     UserID,
 )
+from mautrix.util import background_task
 from mautrix.util.logging import TraceLogger
 from mautrix.util.simple_lock import SimpleLock
 
@@ -402,7 +403,7 @@ class BasePortal(ABC):
         for msg in msgs:
             portal = await cls.bridge.get_portal(msg.room_id)
             if portal and portal.mxid:
-                asyncio.create_task(portal._disappear_event(msg))
+                background_task.create(portal._disappear_event(msg))
             else:
                 await msg.delete()
 
@@ -418,7 +419,7 @@ class BasePortal(ABC):
             for msg in msgs:
                 msg.start_timer()
                 await msg.update()
-                asyncio.create_task(self._disappear_event(msg))
+                background_task.create(self._disappear_event(msg))
 
     async def _send_message(
         self,
@@ -431,7 +432,7 @@ class BasePortal(ABC):
             event_type, content = await self.matrix.e2ee.encrypt(self.mxid, event_type, content)
         event_id = await intent.send_message_event(self.mxid, event_type, content, **kwargs)
         if intent.api.is_real_user:
-            asyncio.create_task(intent.mark_read(self.mxid, event_id))
+            background_task.create(intent.mark_read(self.mxid, event_id))
         return event_id
 
     @property
@@ -458,6 +459,19 @@ class BasePortal(ABC):
         message: str = "Cleaning room",
         puppets_only: bool = False,
     ) -> None:
+        if not puppets_only and cls.bridge.homeserver_software.is_hungry:
+            try:
+                await intent.beeper_delete_room(room_id)
+                return
+            except MNotFound as err:
+                cls.log.debug(f"Hungryserv yeet returned {err}, assuming the room is already gone")
+                return
+            except Exception:
+                cls.log.warning(
+                    f"Failed to delete {room_id} using hungryserv yeet endpoint, "
+                    f"falling back to normal method",
+                    exc_info=True,
+                )
         try:
             members = await intent.get_room_members(room_id)
         except MatrixError:
