@@ -12,8 +12,16 @@ import time
 
 from mautrix import __optional_imports__
 from mautrix.api import MediaPath, Method
-from mautrix.errors import MatrixResponseError
-from mautrix.types import ContentURI, MediaRepoConfig, MXOpenGraph, SerializerError
+from mautrix.errors import MatrixResponseError, make_request_error
+from mautrix.types import (
+    ContentURI,
+    MediaCreateResponse,
+    MediaRepoConfig,
+    MXOpenGraph,
+    SerializerError,
+)
+from mautrix.util import background_task
+from mautrix.util.async_body import async_iter_bytes
 from mautrix.util.opt_prometheus import Histogram
 
 from ..base import BaseClientAPI
@@ -45,22 +53,16 @@ class MediaRepositoryMethods(BaseClientAPI):
     uploads of media.
     """
 
-    async def unstable_create_mxc(self) -> ContentURI:
+    async def unstable_create_mxc(self) -> MediaCreateResponse:
         """
         Create a media ID for uploading media to the homeserver. Requires the homeserver to have
         `MSC2246 <https://github.com/matrix-org/matrix-spec-proposals/pull/2246>`__ support.
 
         Returns:
-            The MXC URI that can be used to upload a file to later.
-
-        Raises:
-            MatrixResponseError: If the response does not contain a ``content_uri`` field.
+            MediaCreateResponse Containing the MXC URI that can be used to upload a file to later, as well as an optional upload URL
         """
         resp = await self.api.request(Method.POST, MediaPath.unstable["fi.mau.msc2246"].create)
-        try:
-            return resp["content_uri"]
-        except KeyError:
-            raise MatrixResponseError("`content_uri` not in response.")
+        return MediaCreateResponse.deserialize(resp)
 
     @contextmanager
     def _observe_upload_time(self, size: int | None, mxc: ContentURI | None = None) -> None:
@@ -121,19 +123,32 @@ class MediaRepositoryMethods(BaseClientAPI):
         if filename:
             query["filename"] = filename
 
+        upload_url = None
+
         if async_upload:
             if mxc:
                 raise ValueError("async_upload and mxc can't be provided simultaneously")
-            mxc = await self.unstable_create_mxc()
+            create_response = await self.unstable_create_mxc()
+            mxc = create_response.content_uri
+            upload_url = create_response.upload_url
 
         path = MediaPath.v3.upload
         method = Method.POST
         if mxc:
             server_name, media_id = self.api.parse_mxc_uri(mxc)
-            path = MediaPath.unstable["fi.mau.msc2246"].upload[server_name][media_id]
-            method = Method.PUT
+            if upload_url is None:
+                path = MediaPath.unstable["fi.mau.msc2246"].upload[server_name][media_id]
+                method = Method.PUT
+            else:
+                path = MediaPath.unstable["fi.mau.msc2246"].upload[server_name][media_id].complete
+
+        if upload_url is not None:
+            task = self._upload_to_url(upload_url, path, headers, data, post_upload_query=query)
+        else:
+            task = self.api.request(
+                method, path, content=data, headers=headers, query_params=query
+            )
 
-        task = self.api.request(method, path, content=data, headers=headers, query_params=query)
         if async_upload:
 
             async def _try_upload():
@@ -143,7 +158,7 @@ class MediaRepositoryMethods(BaseClientAPI):
                 except Exception as e:
                     self.log.error(f"Failed to upload {mxc}: {type(e).__name__}: {e}")
 
-            asyncio.create_task(_try_upload())
+            background_task.create(_try_upload())
             return mxc
         else:
             with self._observe_upload_time(size):
@@ -168,7 +183,7 @@ class MediaRepositoryMethods(BaseClientAPI):
             The raw downloaded data.
         """
         url = self.api.get_download_url(url)
-        query_params: dict[str, Any] = {}
+        query_params: dict[str, Any] = {"allow_redirect": "true"}
         if max_stall_ms is not None:
             query_params["max_stall_ms"] = max_stall_ms
             query_params["fi.mau.msc2246.max_stall_ms"] = max_stall_ms
@@ -206,7 +221,7 @@ class MediaRepositoryMethods(BaseClientAPI):
             The raw downloaded data.
         """
         url = self.api.get_download_url(url, download_type="thumbnail")
-        query_params: dict[str, Any] = {}
+        query_params: dict[str, Any] = {"allow_redirect": "true"}
         if width is not None:
             query_params["width"] = width
         if height is not None:
@@ -265,3 +280,46 @@ class MediaRepositoryMethods(BaseClientAPI):
             return MediaRepoConfig.deserialize(content)
         except SerializerError as e:
             raise MatrixResponseError("Invalid MediaRepoConfig in response") from e
+
+    async def _upload_to_url(
+        self,
+        upload_url: str,
+        post_upload_path: str,
+        headers: dict[str, str],
+        data: bytes | bytearray | AsyncIterable[bytes],
+        post_upload_query: dict[str, str],
+        min_iter_size: int = 25 * 1024 * 1024,
+    ) -> None:
+        retry_count = self.api.default_retry_count
+        backoff = 2
+        do_fake_iter = data and hasattr(data, "__len__") and len(data) > min_iter_size
+        if do_fake_iter:
+            headers["Content-Length"] = str(len(data))
+        while True:
+            self.log.debug("Uploading media to external URL %s", upload_url)
+            upload_response = None
+            try:
+                req_data = async_iter_bytes(data) if do_fake_iter else data
+                upload_response = await self.api.session.put(
+                    upload_url, data=req_data, headers=headers
+                )
+                upload_response.raise_for_status()
+            except Exception as e:
+                if retry_count <= 0:
+                    raise make_request_error(
+                        http_status=upload_response.status if upload_response else -1,
+                        text=(await upload_response.text()) if upload_response else "",
+                        errcode="COM.BEEPER.EXTERNAL_UPLOAD_ERROR",
+                        message=None,
+                    )
+                self.log.warning(
+                    f"Uploading media to external URL {upload_url} failed: {e}, "
+                    f"retrying in {backoff} seconds",
+                )
+                await asyncio.sleep(backoff)
+                backoff *= 2
+                retry_count -= 1
+            else:
+                break
+
+        await self.api.request(Method.POST, post_upload_path, query_params=post_upload_query)
