@@ -31,6 +31,7 @@ from .crypto import (
 from .event import (
     AccountDataEvent,
     AccountDataEventContent,
+    ASToDeviceEvent,
     AudioInfo,
     BaseEvent,
     BaseFileInfo,
@@ -138,7 +139,14 @@ from .event import (
 )
 from .filter import EventFilter, Filter, RoomEventFilter, RoomFilter, StateFilter
 from .matrixuri import IdentifierType, MatrixURI, MatrixURIError, URIAction
-from .media import MediaRepoConfig, MXOpenGraph, OpenGraphAudio, OpenGraphImage, OpenGraphVideo
+from .media import (
+    MediaCreateResponse,
+    MediaRepoConfig,
+    MXOpenGraph,
+    OpenGraphAudio,
+    OpenGraphImage,
+    OpenGraphVideo,
+)
 from .misc import (
     BatchSendResponse,
     DeviceLists,
@@ -224,6 +232,7 @@ __all__ = [
     "UnsignedDeviceInfo",
     "AccountDataEvent",
     "AccountDataEventContent",
+    "ASToDeviceEvent",
     "AudioInfo",
     "BaseEvent",
     "BaseFileInfo",
@@ -337,6 +346,7 @@ __all__ = [
     "MatrixURI",
     "MatrixURIError",
     "URIAction",
+    "MediaCreateResponse",
     "MediaRepoConfig",
     "MXOpenGraph",
     "OpenGraphAudio",
