@@ -17,7 +17,7 @@ from .encrypted import EncryptedOlmEventContent, EncryptionAlgorithm
 class RoomKeyWithheldCode(ExtensibleEnum):
     BLACKLISTED: "RoomKeyWithheldCode" = "m.blacklisted"
     UNVERIFIED: "RoomKeyWithheldCode" = "m.unverified"
-    UNAUTHORIZED: "RoomKeyWithheldCode" = "m.unauthorized"
+    UNAUTHORIZED: "RoomKeyWithheldCode" = "m.unauthorised"
     UNAVAILABLE: "RoomKeyWithheldCode" = "m.unavailable"
     NO_OLM_SESSION: "RoomKeyWithheldCode" = "m.no_olm"
 
@@ -100,7 +100,9 @@ class ToDeviceEvent(BaseEvent, SerializableAttrs):
             data.setdefault("content", {})["__mautrix_event_type"] = evt_type
         except ValueError:
             return Obj(**data)
-        return super().deserialize(data)
+        evt = super().deserialize(data)
+        evt.type = evt_type
+        return evt
 
     @staticmethod
     @deserializer(ToDeviceEventContent)
@@ -110,3 +112,9 @@ class ToDeviceEvent(BaseEvent, SerializableAttrs):
         if not content_type:
             return Obj(**data)
         return content_type.deserialize(data)
+
+
+@dataclass
+class ASToDeviceEvent(ToDeviceEvent, SerializableAttrs):
+    to_user_id: UserID
+    to_device_id: DeviceID
